package com.gic.plug.web.controller;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.enterprise.constants.Constants;
import com.gic.enterprise.utils.UserDetailUtils;
import com.gic.goods.api.util.Constant;
import com.gic.plug.web.vo.WidgetValuesNewVO;
import com.gic.plug.web.vo.WidgetValuesOldVO;
import com.gic.widget.screening.api.dto.EsScreeningChainDetailDTO;
import com.gic.widget.screening.api.dto.EsScreeningEnterpriseCategorySceneInfoDTO;
import com.gic.widget.screening.api.dto.EsScreeningTemplateDTO;
import com.gic.widget.screening.api.service.EsScreeningInitService;
import com.gic.widget.screening.api.service.EsScreeningTemplateService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;


/**
 * 门店表情
 * @ClassName: ScreeningController 
 * @Description:  
 * @author boom 
 * @date 2020/9/14 10:36 AM 
 */
@Controller
public class ScreeningController {

    private static final Logger LOGGER = LogManager.getLogger(ScreeningController.class);

    @Autowired
    private EsScreeningInitService esScreeningInitService;
    @Autowired
    private EsScreeningTemplateService esScreeningTemplateService;

    /**
     * @return
     * @throws
     * @Title: getScreeningTemplate
     * @Description: 获取模板
     * @author xugh
     */
    @RequestMapping("get-screening-template")
    @ResponseBody
    public RestResponse getScreeningTemplate() {
        List<EsScreeningTemplateDTO> list = this.esScreeningTemplateService.queryEsScreeningTemplateList();
        return RestResponse.success(list);
    }

    /**
     * @param sceneCode
     * @return
     * @throws
     * @Title: getScreeningInitData
     * @Description: 获取初始化筛选条件
     * @author xugh
     */
    @RequestMapping("get-screening-init-data")
    @ResponseBody
    public RestResponse getScreeningInitData(@RequestParam(defaultValue = "member") String sceneCode) {
        String enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId().toString();
        if (Constants.STORE_SCENECODE.equals(sceneCode)) {
            enterpriseId = Constants.INDEX_ENTERPRISEID;
        }
        if (Constant.ERP_GOODS_SCREEN.equals(sceneCode) || Constant.WEIMOB_GOODS_SCREEN.equals(sceneCode) || Constant.CHANNEL_CODE_MALL.equals(sceneCode)) {
            //TODO
            String testEnterpriseId = "1129";
            if (testEnterpriseId.equals(enterpriseId)) {
                enterpriseId = "goods-test-1129";
            }
        }
        List<EsScreeningEnterpriseCategorySceneInfoDTO> list = this.esScreeningInitService
                .queryEsScreeningByEnterpriseAndScene(enterpriseId, sceneCode);
        Collections.sort(list, new Comparator<EsScreeningEnterpriseCategorySceneInfoDTO>() {
            @Override
            public int compare(EsScreeningEnterpriseCategorySceneInfoDTO o1, EsScreeningEnterpriseCategorySceneInfoDTO o2) {
                return o1.getCreateTime().compareTo(o2.getCreateTime());
            }
        });
        List<EsScreeningChainDetailDTO> all = new ArrayList<>();
        for (EsScreeningEnterpriseCategorySceneInfoDTO dto : list) {
            String categoryId = dto.getSceneCategoryId();
            EsScreeningChainDetailDTO item = this.esScreeningInitService.findRootChainByCategoryId(categoryId);
            String widgetValues = item.getWidget().getWidgetValues();
            if (StringUtils.isNotBlank(widgetValues)) {
                List<WidgetValuesOldVO> oldVOS = JSONObject.parseArray(widgetValues, WidgetValuesOldVO.class);
                List<WidgetValuesNewVO> newVOList = oldVOS.stream().map(data -> new WidgetValuesNewVO(data.getKey(), data.getValue(), data.getName())).collect(Collectors.toList());
                item.getWidget().setWidgetValues(JSON.toJSONString(newVOList));
            }
            all.add(item);
        }
        return RestResponse.success(all);
    }

    @RequestMapping("get-screening-data")
    @ResponseBody
    public RestResponse getScreeningData(String widgetChainId) {
        List<EsScreeningChainDetailDTO> list = this.esScreeningInitService
                .queryEsScreeningByParentChainId(widgetChainId);
        if (CollectionUtils.isNotEmpty(list)) {
            list = list.stream().peek(item -> {
                if (StringUtils.isNotBlank(item.getWidget().getWidgetValues())) {
                    List<WidgetValuesOldVO> oldVOS = JSONObject.parseArray(item.getWidget().getWidgetValues(), WidgetValuesOldVO.class);
                    List<WidgetValuesNewVO> newVOList = oldVOS.stream().map(data -> new WidgetValuesNewVO(data.getKey(), data.getValue(), data.getName())).collect(Collectors.toList());
                    item.getWidget().setWidgetValues(JSON.toJSONString(newVOList));
                }
            }).collect(Collectors.toList());
        }
        return RestResponse.success(list);
    }

    @RequestMapping("get-screening-chain-detail")
    @ResponseBody
    public RestResponse getScreeningChainDetail(String parentChainId, String widgetFieldKey) {
        EsScreeningChainDetailDTO dto = this.esScreeningInitService.findChainDetail(parentChainId, widgetFieldKey);
        if (dto != null && dto.getWidget() != null && StringUtils.isNotBlank(dto.getWidget().getWidgetValues())) {
            List<WidgetValuesOldVO> oldVOS = JSONObject.parseArray(dto.getWidget().getWidgetValues(), WidgetValuesOldVO.class);
            List<WidgetValuesNewVO> newVOList = oldVOS.stream().map(data -> new WidgetValuesNewVO(data.getKey(), data.getValue(), data.getName())).collect(Collectors.toList());
            dto.getWidget().setWidgetValues(JSON.toJSONString(newVOList));
        }
        return RestResponse.success(dto);
    }


    @RequestMapping("get-screening-widget-chain-detail")
    @ResponseBody
    public Object getScreeningWidgetDetail(String widgetChainId) {
        EsScreeningChainDetailDTO dto = this.esScreeningInitService.findChainDetail(widgetChainId);
        if (dto != null && dto.getWidget() != null && StringUtils.isNotBlank(dto.getWidget().getWidgetValues())) {
            List<WidgetValuesOldVO> oldVOS = JSONObject.parseArray(dto.getWidget().getWidgetValues(), WidgetValuesOldVO.class);
            List<WidgetValuesNewVO> newVOList = oldVOS.stream().map(data -> new WidgetValuesNewVO(data.getKey(), data.getValue(), data.getName())).collect(Collectors.toList());
            dto.getWidget().setWidgetValues(JSON.toJSONString(newVOList));
        }
        return RestResponse.success(dto);
    }
}
