package com.gic.plug.web.controller;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

import com.gic.api.base.commons.Page;
import com.gic.plug.web.vo.widget.StoreWidgetInterfaceVO;
import com.gic.store.dto.StoreDTO;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.dto.ResourceGroupDTO;
import com.gic.auth.dto.UserResourceDTO;
import com.gic.auth.service.ResourceGroupApiService;
import com.gic.auth.service.UserResourceApiService;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.enterprise.utils.ResultControllerUtils;
import com.gic.enterprise.utils.UserDetail;
import com.gic.enterprise.utils.UserDetailUtils;
import com.gic.plug.web.qo.StoreWidgetCountQO;
import com.gic.plug.web.qo.StoreWidgetQO;
import com.gic.plug.web.strategy.init.StrategyInit;
import com.gic.plug.web.utils.StoreResourceAuthUtils;
import com.gic.plug.web.utils.storeresourceauth.StoreResourceUtils;
import com.gic.plug.web.vo.*;
import com.gic.store.constant.StoreESFieldsEnum;
import com.gic.store.dto.StoreRegionDTO;
import com.gic.store.dto.StoreSearchDTO;
import com.gic.store.dto.StoreWidgetDTO;
import com.gic.store.service.StoreApiService;
import com.gic.store.service.StoreRegionApiService;
import com.gic.store.service.StoreWidgetApiService;
import com.gic.store.service.StoreWidgetLogApiService;

/**
 * @author zhiwj
 * @date 2019/7/12
 */
@RestController
public class StoreController {

    @Autowired
    private StoreRegionApiService storeRegionApiService;

    @Autowired
    private StoreApiService storeApiService;
    @Autowired
    private StoreWidgetApiService storeWidgetApiService;
    @Autowired
    private StrategyInit strategyInit;
    @Autowired
    private UserResourceApiService userResourceApiService;
    @Autowired
    private StoreWidgetLogApiService storeWidgetLogApiService;
    @Autowired
    private ResourceGroupApiService resourceGroupApiService;

    @RequestMapping("/list-store-region")
    public RestResponse listStoreRegion(String search) {
        UserDetail userDetail = UserDetailUtils.getUserDetail();
        Integer enterpriseId = userDetail.getEnterpriseId();
        Set<String> authRegionSet = StoreResourceAuthUtils.getAuthData(StoreESFieldsEnum.REGIONID.getField());
        ServiceResponse<List<StoreRegionDTO>> serviceResponse = storeRegionApiService.listStoreRegion(enterpriseId, search);
        if (serviceResponse.isSuccess()) {
            List<StoreRegionDTO> result = serviceResponse.getResult();

            List<StoreWidgetInterfaceVO> voList = new ArrayList<>();
            for(StoreRegionDTO storeRegionDTO : result){
                if (!authRegionSet.isEmpty()) {
                    if(authRegionSet.contains(storeRegionDTO.getRegionId()+"")){
                        voList.add(new StoreWidgetInterfaceVO().setLabel(storeRegionDTO.getRegionName()).setValue(storeRegionDTO.getRegionId().toString()));
                    }
                 }else {
                    voList.add(new StoreWidgetInterfaceVO().setLabel(storeRegionDTO.getRegionName()).setValue(storeRegionDTO.getRegionId().toString()));
                }
            }
            return RestResponse.success(voList);
        } else {
            return RestResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
        }
    }

    @RequestMapping("/list-part-store")
    public RestResponse listPartStore(Integer storeGroupId, String search) {
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        StoreSearchDTO storeSearchDTO = new StoreSearchDTO();

        //权限过滤部分门店
        List<Integer> authStorePart = StoreResourceUtils.auth(null, StoreESFieldsEnum.STOREID.getField());
        if (CollectionUtils.isNotEmpty(authStorePart)) {
            StringBuilder storeIds = new StringBuilder();
            authStorePart.forEach(e -> {
                storeIds.append(e).append(" ");
            });
            storeSearchDTO.setStoreIds(storeIds.toString());
        }

        storeSearchDTO.setEnterpriseId(enterpriseId);
        storeSearchDTO.setStoreGroupId(storeGroupId);
        storeSearchDTO.setSearch(search);
        storeSearchDTO.setStoreResource(UserDetailUtils.getUserDetail().getStoreResourceId());
        String returnFields = StoreESFieldsEnum.STOREID.getField()+","+StoreESFieldsEnum.STORECODE.getField()+","+StoreESFieldsEnum.STORENAME.getField();
        ServiceResponse<Page<StoreDTO>> serviceResponse = storeApiService.listStore(storeSearchDTO,
                1, 100, returnFields);
        if (serviceResponse.isSuccess()) {
            Page<StoreDTO> page = serviceResponse.getResult();
            List<StoreDTO> list = page.getResult();

            List<StoreWidgetInterfaceVO> voList = new ArrayList<>();
            if (CollectionUtils.isNotEmpty(list)) {
                if (CollectionUtils.isNotEmpty(list)) {
                    voList = list.stream()
                            .map(e -> new StoreWidgetInterfaceVO().setLabel(e.getStoreName()).setValue(e.getStoreId().toString()))
                            .collect(Collectors.toList());
                }
            }
            return RestResponse.success(voList);
        } else {
            return RestResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
        }
    }

    @RequestMapping("get-store-count")
    public RestResponse getStoreCount(@RequestBody StoreWidgetCountQO storeWidgetCountQO){
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        StoreSearchDTO storeSearchDTO = new StoreSearchDTO();
        storeSearchDTO.setEnterpriseId(enterpriseId);
        storeSearchDTO.setSearchJson(storeWidgetCountQO.getSearchJson());
        storeSearchDTO.setAuthSearchJson(storeWidgetCountQO.getAuthSearchJson());
        ServiceResponse<Long> response = storeApiService.queryStoreCountFromEs(storeSearchDTO);
        return RestResponse.success(response.getResult());
    }

    /**
     * 更新权限范围
     * @param storeWidgetId
     * @return
     */
    @RequestMapping("update-auth-search-param")
    public RestResponse updateAuth(Integer storeWidgetId) {
        StoreWidgetDTO storeWidgetDTO = new StoreWidgetDTO();
        storeWidgetDTO.setStoreWidgetId(storeWidgetId);
        storeWidgetDTO.setUserId(UserDetailUtils.getUserDetail().getUserId());
        ServiceResponse<Integer> response = storeWidgetApiService.updateAuth(storeWidgetDTO);
        if (response.isSuccess()) {
            return RestResponse.success();
        }
        return RestResponse.failure(response.getCode(), response.getMessage());
    }

    @RequestMapping("save-store-widget")
    public RestResponse saveStoreWidget(@RequestBody StoreWidgetQO storeWidgetQO){
        //参数组装
        StoreWidgetDTO storeWidgetDTO = new StoreWidgetDTO();

        storeWidgetDTO.setSearchParam(storeWidgetQO.getSearchParams());
        storeWidgetDTO.setStoreWidgetId(storeWidgetQO.getStoreWidgetId());
        storeWidgetDTO.setWriteBackParam(storeWidgetQO.getWriteBackParam());
        storeWidgetDTO.setAuthMode(storeWidgetQO.getAuthMode());
        storeWidgetDTO.setScreenDetailData(storeWidgetQO.getScreenDetailData());
        storeWidgetDTO.setScreenSimpleData(storeWidgetQO.getScreenSimpleData());
        storeWidgetDTO.setUserId(UserDetailUtils.getUserDetail().getUserId());

        ServiceResponse<Integer> storeWidgetResponse = this.storeWidgetApiService.saveAuthStoreWidget(storeWidgetDTO);
        if (storeWidgetResponse.isSuccess()) {
            return RestResponse.success(storeWidgetResponse.getResult());
        }
        return RestResponse.success();
    }

    /**
     * 门店选择器，编辑的时候，获取最新一名操作人信息
     * @param storeWidgetId
     * @return
     */
    @RequestMapping("get-store-widget-last-operator")
    public RestResponse getStoreWidgetLastOperator(Integer storeWidgetId) {
        return ResultControllerUtils.commonResultOne(storeWidgetLogApiService.getLastOperationInfo(storeWidgetId), StoreWidgetLogVO.class);
    }

    /**
     * 根据用户ID查询对应的资源组-门店选择器数据
     * @param userId
     * @return
     * @throws ClassNotFoundException
     */
    @RequestMapping("get-user-store-widget-resource")
    public RestResponse getUserStoreWidgetResource(Integer userId) throws ClassNotFoundException{
        return RestResponse.success(getStoreResourceByUser(userId));
    }

    /**
     * 查询门店选择器的操作日志
     * @param storeWidgetId
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping("list-store-widget-log")
    public RestResponse listStoreWidgetLog(Integer storeWidgetId, Integer pageNum, Integer pageSize) {
        return ResultControllerUtils
                .commonPageResult(storeWidgetLogApiService.pageStoreWidgetLog(storeWidgetId, pageNum, pageSize), StoreWidgetLogVO.class);
    }


    @RequestMapping("get-store-widget")
    public RestResponse getStoreWidget(Integer storeWidgetId) throws ClassNotFoundException {
        ServiceResponse<StoreWidgetDTO> response = this.storeWidgetApiService.getStoreWidget(storeWidgetId);
        StoreWidgetVO storeWidgetVO = EntityUtil.changeEntityByJSON(StoreWidgetVO.class, response.getResult());
//        storeWidgetVO.setScreenBack(this.getScreenBack(storeWidgetVO.getSearchParam()));
        return RestResponse.success(storeWidgetVO);
    }

    @RequestMapping("screen-back")
    @ResponseBody
    public Object screenBack(String param) throws ClassNotFoundException {
        return RestResponse.success();
    }

    @RequestMapping("get-user-store-resource")
    @ResponseBody
    public RestResponse getUserStoreResource() throws ClassNotFoundException {
        //管理员
        if (UserDetailUtils.getUserDetail().getUserInfo().getSuperAdmin() != 1) {
            return RestResponse.success(getStoreResourceByUser(UserDetailUtils.getUserDetail().getUserId()));
        }
        //超管
        return RestResponse.success(new StoreResourceVO());
    }

    private StoreResourceVO getStoreResourceByUser(Integer userId) throws ClassNotFoundException {
        //查询管理员的资源组
        ServiceResponse<UserResourceDTO> resource = this.userResourceApiService
                .getResourceByUserId(userId);
        if (resource.isSuccess() && resource.getResult() != null) {
            ServiceResponse<ResourceGroupDTO> serviceResource = this.resourceGroupApiService
                    .getResourceGroup(resource.getResult().getResourceId());
            //资源组-门店选择器ID
            if(serviceResource.isSuccess() && serviceResource.getResult() != null){
                ServiceResponse<StoreWidgetDTO> storeWidget = this.storeWidgetApiService
                        .getStoreWidget(serviceResource.getResult().getStoreResource().intValue());
                if(storeWidget.isSuccess()){
                    StoreWidgetDTO storeWidgetDTO = storeWidget.getResult();

                    StoreResourceVO vo = EntityUtil.changeEntityNew(StoreResourceVO.class, storeWidgetDTO);
                    vo.setSearchJson(storeWidgetDTO.getSearchParam());
//                    vo.setSceenBack(getScreenBack(storeWidgetDTO.getSearchParam()));
                    return vo;
                }
            }
        }
        return new StoreResourceVO();
    }

    private List<ScreenBackVO> getScreenBack(String param) throws ClassNotFoundException {
        if(StringUtils.isBlank(param)){
            return null;
        }
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        JSONArray array = JSON.parseArray(param);
        List<ScreenBackVO> result = new ArrayList<>();
        for(int i=0; i< array.size(); i++){
            JSONObject jsonObject = array.getJSONObject(i);
            ScreenBackVO screenBackVO = new ScreenBackVO();
            List<ScreenBack> data = new ArrayList<>();
            String type = jsonObject.getString("type");
            if(StringUtils.isNotBlank(type)){
               screenBackVO.setType(type);
            }
            JSONArray list = jsonObject.getJSONArray("list");
            for(int j = 0; j<list.size(); j++){
                JSONObject json = list.getJSONObject(j);
                String key = json.getJSONObject("data").getString("key");
                String value = json.getJSONObject("data").getString("value");
                String compute = json.getJSONObject("data").getString("compute");
                ScreenBack screenBack = this.strategyInit.getStrategyMap().get(key).screenBack(compute, value, key, enterpriseId);
                data.add(screenBack);
            }
            screenBackVO.setData(data);
            result.add(screenBackVO);
        }
        return result;
    }
}
