package com.gic.plug.web.controller;

import java.util.*;
import java.util.stream.Collectors;

import com.gic.plug.web.utils.StoreRegionAuthUtil;
import com.gic.plug.web.utils.StoreResourceAuthUtils;
import com.gic.plug.web.utils.storeresourceauth.StoreResourceUtils;
import com.gic.plug.web.vo.widget.StoreWidgetInterfaceVO;
import com.gic.store.constant.StoreESFieldsEnum;
import com.gic.store.service.StoreWidgetApiService;
import com.gic.store.utils.StoreRedisKeyUtils;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.utils.UserDetailUtils;
import com.gic.plug.web.vo.StoreFieldRegionVO;
import com.gic.plug.web.vo.StoreFieldSelectVO;
import com.gic.redis.data.util.RedisUtil;
import com.gic.store.constant.StoreFieldDateEnum;
import com.gic.store.constant.StoreFieldTextEnum;
import com.gic.store.constant.StoreFieldTypeEnum;
import com.gic.store.dto.StoreFieldDTO;
import com.gic.store.dto.StoreFieldSelectDTO;
import com.gic.store.service.StoreFieldApiService;
import com.gic.store.service.StoreFieldSelectApiService;

/**
 * @author guojx
 * @date 2019/7/15 9:21 AM
 */
@RestController
@RequestMapping("/store-field")
public class StoreFieldController {
    private static final Logger logger = LoggerFactory.getLogger(StoreFieldController.class);

    @Autowired
    private StoreFieldApiService storeFieldApiService;
    @Autowired
    private StoreFieldSelectApiService storeFieldSelectApiService;
    @Autowired
    private StoreWidgetApiService widgetApiService;
    @Autowired
    private StoreWidgetApiService storeWidgetApiService;

    @RequestMapping("/list-store-field-type")
    public RestResponse listStoreFieldType() {
        Map<Integer, String> result = new HashMap<>(16);
        for (StoreFieldTypeEnum typeEnum : StoreFieldTypeEnum.values()) {
            result.put(typeEnum.getCode(), typeEnum.getMessage());
        }
        return RestResponse.success(result);
    }

    @RequestMapping("/list-store-field-date-type")
    public RestResponse listStoreFieldDateType() {
        Map<Integer, String> result = new HashMap<>(16);
        for (StoreFieldDateEnum typeEnum : StoreFieldDateEnum.values()) {
            result.put(typeEnum.getCode(), typeEnum.getMessage());
        }
        return RestResponse.success(result);
    }

    /**
     * 文本类型
     * @return
     */
    @RequestMapping("/list-store-field-text-type")
    public RestResponse listStoreFieldTextType() {
        Map<Integer, String> result = new HashMap<>(16);
        for (StoreFieldTextEnum typeEnum : StoreFieldTextEnum.values()) {
            result.put(typeEnum.getCode(), typeEnum.getMessage());
        }
        return RestResponse.success(result);
    }

    @RequestMapping("/list-store-field-by-region-id")
    public RestResponse listStoreFieldByRegionId(Integer regionId) {
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        if (regionId == null) {
            RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "域ID不能为空");
        }
        ServiceResponse<List<StoreFieldDTO>> result = storeFieldApiService.listStoreFieldByRegionId(regionId);
        if (result.isSuccess()) {
            return RestResponse.success(transferDtoToVo(result.getResult(), enterpriseId));
        } else {
            return RestResponse.failure(result.getCode(), result.getMessage());
        }
    }

    @RequestMapping("/list-store-field")
    public RestResponse listStoreField(String search) {
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        ServiceResponse<List<StoreFieldDTO>> result = storeFieldApiService.listStoreField(enterpriseId, search);
        if (result.isSuccess()) {
            List<StoreFieldRegionVO> dealResult = transferDtoToVo(result.getResult(), enterpriseId);
            List<StoreWidgetInterfaceVO> voList = new ArrayList<>();
            if (CollectionUtils.isNotEmpty(dealResult)) {
                voList = dealResult.stream()
                        .map(e -> new StoreWidgetInterfaceVO()
                                .setLabel(e.getStoreFieldName())
                                .setValue(e.getStoreFieldKey().toString())
                                .setKey(getFieldKeyByFieldType(e.getStoreFieldType())))
                        .collect(Collectors.toList());
            }
            return RestResponse.success(voList);
        } else {
            return RestResponse.failure(result.getCode(), result.getMessage());
        }
    }

    @RequestMapping("/list-store-field-select")
    public RestResponse listStoreFieldSelect(String search, Integer storeFieldId) {
        ServiceResponse<List<StoreFieldSelectDTO>> result = storeFieldSelectApiService
                .listStoreFieldSelect(storeFieldId, search);
        if (result.isSuccess()) {
            List<StoreFieldSelectDTO> list = result.getResult();
            List<StoreWidgetInterfaceVO> voList = new ArrayList<>();
            if (CollectionUtils.isNotEmpty(list)) {
                voList = list.stream()
                        .map(e -> new StoreWidgetInterfaceVO().setValue(e.getStoreFieldSelectId().toString()).setLabel(e.getStoreFieldSelectName()))
                        .collect(Collectors.toList());
            }
            return RestResponse.success(voList);
        } else {
            return RestResponse.failure(result.getCode(), result.getMessage());
        }
    }

    private List<StoreFieldRegionVO> transferDtoToVo(List<StoreFieldDTO> dtoList, Integer enterpriseId) {
        //自定义字段权限过滤数据
        List<StoreFieldDTO> storeFieldAuthData = StoreResourceUtils
                .auth(dtoList, StoreResourceUtils.STORE_FIELD_KEY);

        List<StoreFieldRegionVO> voList = new ArrayList<>(storeFieldAuthData.size());
        Set<String> userRegion = StoreResourceAuthUtils.getAuthData(StoreESFieldsEnum.REGIONID.getField());
        for (StoreFieldDTO dto : storeFieldAuthData) {
            //不需要文本类型字段
            if(!userRegion.isEmpty()){
                if(!userRegion.contains(dto.getStoreRegionId()+"")){
                    continue;
                }
            }
            if (dto.getStoreFieldType().intValue() != StoreFieldTypeEnum.TEXT.getCode()) {
                Object obj = RedisUtil.getCache(StoreRedisKeyUtils.getStoreFieldKey(enterpriseId, dto.getStoreFieldId()));
                if (obj != null) {
                    StoreFieldRegionVO vo = EntityUtil.changeEntityNew(StoreFieldRegionVO.class, dto);
                    vo.setStoreFieldKey(obj.toString());
                    if (vo.getStoreFieldType().intValue() == StoreFieldTypeEnum.DATE.getCode()) {
                        JSONObject json = JSON.parseObject(dto.getStoreFieldDetail());
                        vo.setDateType(json.getIntValue("dateType"));
                    }
                    voList.add(vo);
                }
            }
        }
        return voList;
    }

    private String getFieldKeyByFieldType(Integer fieldType) {
        if (fieldType == null) {
            return null;
        }
        StoreFieldTypeEnum typeEnum = StoreFieldTypeEnum.getByCode(fieldType);
        switch (typeEnum) {
            case DATE:
                return "com006";
            case CHECK:
                return "com007";
            case RADIO:
                return "com009";
            default:
                return null;

        }
    }
}
