package com.gic.plug.web.controller;

import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.enterprise.utils.UserDetailUtils;
import com.gic.store.constant.StoreGroupConstant;
import com.gic.store.dto.StoreGroupDTO;
import com.gic.store.service.StoreGroupApiService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author guojx
 * @date 2019/7/12 2:54 PM
 */
@RestController
@RequestMapping("/store-group")
public class StoreGroupController {
    private static final Logger logger = LoggerFactory.getLogger(StoreGroupController.class);

    @Autowired
    private StoreGroupApiService storeGroupApiService;

    @RequestMapping("/list")
    public RestResponse listStoreGroup() {
        StoreGroupDTO dto = new StoreGroupDTO();
        dto.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        List<StoreGroupDTO> list = storeGroupApiService.listStoreGroup(dto).getResult();
        return RestResponse.success(changeListToTreeNotIncludeAllStore(StoreGroupConstant.ALL_STORE_LEVEL, list));
    }

    /**
     * 把集合数据成树结构
     * @Title: changeListToTree 
     * @Description:
      * @author guojuxing
     * @param storeGroupId
     * @param list 
     * @return  java.util.List<java.util.Map<java.lang.String,java.lang.Object>> 
      */
    private static List<Map<String, Object>> changeListToTree(int storeGroupId, List<StoreGroupDTO> list) {
        List<Map<String, Object>> result = new ArrayList<>();
        if (list != null) {
            for (StoreGroupDTO t : list) {
                if (storeGroupId == t.getParentStoreGroupId()) {
                    Map<String, Object> data = new HashMap<>(16);
                    data.put("value", t.getStoreGroupId());
                    data.put("label", t.getStoreGroupName());
                    data.put("groupLevel", t.getGroupLevel());
                    List<Map<String, Object>> children = changeListToTree(t.getStoreGroupId(), list);
                    if (children == null || children.size() < 1) {
                        data.put("children", null);
                    } else {
                        data.put("children", children);
                    }
                    result.add(data);
                }
            }
        }
        return result;
    }

    /**
     * 不包括全部门店的数据的树结构数据
     * @Title: changeListToTreeNotIncludeAllStore 
     * @Description:
      * @author guojuxing
     * @param storeGroupId
     * @param list 
     * @return  java.util.List<java.util.Map<java.lang.String,java.lang.Object>> 
      */
    private static List<Map<String, Object>> changeListToTreeNotIncludeAllStore(int storeGroupId,
                                                                               List<StoreGroupDTO> list) {
        List<Map<String, Object>> result = changeListToTree(storeGroupId, list);
        if (result.size() > 0) {
            for (Map<String, Object> obj : result) {
                if ((int) obj.get("groupLevel") == StoreGroupConstant.ALL_STORE_LEVEL) {
                    return (List<Map<String, Object>>) obj.get("children");
                }
            }
        }
        return result;
    }
}
