package com.gic.plug.web.controller.goods;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.contants.SortType;
import com.gic.commons.util.PageHelperUtils;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.enterprise.utils.UserDetailUtils;
import com.gic.goods.api.dto.BrandDTO;
import com.gic.goods.api.dto.GoodsRightsSelectorDTO;
import com.gic.goods.api.dto.GoodsSelectorDTO;
import com.gic.goods.api.service.BrandApiService;
import com.gic.goods.api.service.GoodsRightsSelectorApiService;
import com.gic.goods.api.service.GoodsSelectorApiService;
import com.gic.goods.api.util.Constant;
import com.gic.plug.web.vo.goods.GoodsBrandVO;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
/**
 * 商品
 * @ClassName: GoodsBrandController 
 * @Description:  
 * @author guojuxing 
 * @date 2020/9/14 10:43 AM 
 */
@RestController
public class GoodsBrandController extends BaseGoodsController {

    @Autowired
    private BrandApiService brandApiService;

    @Autowired
    private GoodsRightsSelectorApiService goodsRightsSelectorApiService;

    @Autowired
    private GoodsSelectorApiService goodsSelectorApiService;

    private static final int DEFAULT_PAGE_SIZE = 999;

    @RequestMapping("/goods-brand-list")
    public RestResponse goodsBrandList(int pageSize, int currentPage, String search, Long goodsDomainId, String channelCode, Long goodsSelectorId) {
        pageSize = DEFAULT_PAGE_SIZE;
        String goodsRightsBrands = null;
        if (goodsSelectorId != null) {
            ServiceResponse<GoodsSelectorDTO> selectorServiceResponse = goodsSelectorApiService
                    .getGoodsSelector(goodsSelectorId, getEnterpriseId());
            if (selectorServiceResponse.getResult() != null) {
                goodsRightsBrands = selectorServiceResponse.getResult().getGoodsRightsBrands();
            } else {
                return RestResponse.success(Collections.EMPTY_LIST);
            }
        } else {
            Long resourceId = UserDetailUtils.getUserDetail().getGoodsResourceInfo() == null ? null :
                    UserDetailUtils.getUserDetail().getGoodsResourceInfo().getGoodsResourceId();
            ServiceResponse<GoodsRightsSelectorDTO> selectorDtoServiceResponse = goodsRightsSelectorApiService
                    .getGoodsRightsSelector(resourceId, getEnterpriseId()
                            , UserDetailUtils.getUserDetail().getUserInfo().getSuperAdmin());
            if (!selectorDtoServiceResponse.isSuccess() || selectorDtoServiceResponse.getResult() == null ||
                    selectorDtoServiceResponse.getResult().getHasRights() == Constant.NO) {
                return RestResponse.success(Collections.EMPTY_LIST);
            }
            goodsRightsBrands = selectorDtoServiceResponse.getResult().getGoodsRightsBrands();
        }
        // 是否最高权限
        List<Long> branIdList = new ArrayList<>();
        if (StringUtils.isNotBlank(goodsRightsBrands)) {
            JSONObject brandRightsObject = JSONObject.parseObject(goodsRightsBrands);
            JSONArray array = brandRightsObject.getJSONArray(channelCode + "_" + goodsDomainId);
            if (CollectionUtils.isNotEmpty(array)) {
                for (int i = 0; i < array.size(); i++) {
                    Long brandId = array.getLong(i);
                    branIdList.add(brandId);
                }
            }
        }
        return findBrandListByParams(currentPage, pageSize, goodsDomainId, search, getEnterpriseId());
    }


    /**
     * 权限商品筛选器的品牌接口
     * @param pageSize
     * @param currentPage
     * @param search
     * @param goodsDomainId
     * @return
     */
    @RequestMapping("/goods-brand-list-for-rights")
    public RestResponse goodsBrandListForRights(int pageSize, int currentPage, String search, Long goodsDomainId) {
        return findBrandListByParams(currentPage, pageSize,goodsDomainId, search, getEnterpriseId());
    }

    private RestResponse findBrandListByParams(int currentPage, int pageSize, Long goodsDomainId, String search, Integer enterpriseId) {
        pageSize = DEFAULT_PAGE_SIZE;
        List<Long> branIdList = new ArrayList<>();
        ServiceResponse<Page<BrandDTO>> serviceResponse = brandApiService.findBrandPage(currentPage, pageSize, goodsDomainId, search, branIdList, SortType.sortDescType, enterpriseId);
        if (!serviceResponse.isSuccess()) {
            return RestResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
        }
        Page<GoodsBrandVO> page = PageHelperUtils.changePageToCurrentPage(serviceResponse.getResult(), GoodsBrandVO.class);
        //TODO 品牌id的拼写错误
        for (int i = 0; i < page.getResult().size(); i++) {
            GoodsBrandVO goodsBrandVO = page.getResult().get(i);
            goodsBrandVO.setSpecBrandId(serviceResponse.getResult().getResult().get(i).getSepcBrandId());
        }
        return RestResponse.success(page);
    }

}
