package com.gic.plug.web.controller.goods;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.enterprise.utils.UserDetailUtils;
import com.gic.goods.api.dto.GoodsRightsSelectorDTO;
import com.gic.goods.api.dto.GoodsSelectorDTO;
import com.gic.goods.api.service.GoodsRightsSelectorApiService;
import com.gic.goods.api.service.GoodsSelectorApiService;
import com.gic.mall.base.api.enums.ErrorCodeEnums;
import com.gic.plug.web.qo.goods.GoodsSelectorSaveQO;
import com.gic.plug.web.qo.goods.GoodsSelectorUpdateQO;
import com.gic.plug.web.vo.goods.GoodsSelectorVO;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
/**
 * 商品
 * @ClassName: GoodsSelectorController 
 * @Description:  
 * @author guojuxing 
 * @date 2020/9/14 10:55 AM 
 */
@RestController
public class GoodsSelectorController extends BaseGoodsController {

    @Autowired
    private GoodsSelectorApiService goodsSelectorApiService;

    @Autowired
    private GoodsRightsSelectorApiService goodsRightsSelectorApiService;

    private Logger log = LogManager.getLogger(GoodsSelectorController.class);

    @RequestMapping("goods-selector-show-back")
    public RestResponse goodsSelectorShowBack(Long goodsSelectorId) {
        ServiceResponse<GoodsSelectorDTO> serviceResponse = goodsSelectorApiService.getGoodsSelector(goodsSelectorId, getEnterpriseId());
        if (serviceResponse.isSuccess()) {
            return RestResponse.success(serviceResponse.getResult().getGoodsSearchValue());
        }
        return RestResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
    }

    @RequestMapping("get-goods-selector")
    public RestResponse getGoodsSelector(Long goodsSelectorId) {
        ServiceResponse<GoodsSelectorDTO> serviceResponse = goodsSelectorApiService.getGoodsSelector(goodsSelectorId, getEnterpriseId());
        if (serviceResponse.isSuccess()) {
            GoodsSelectorVO goodsSelectorVO = EntityUtil.changeEntityByOrika(GoodsSelectorVO.class, serviceResponse.getResult());
            return RestResponse.success(goodsSelectorVO);
        }
        return RestResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
    }


    @RequestMapping("goods-selector-edit-show-back")
    public RestResponse goodsSelectorEditShowBack(Long goodsSelectorId) {
        ServiceResponse<GoodsSelectorDTO> serviceResponse = goodsSelectorApiService.getGoodsSelector(goodsSelectorId, getEnterpriseId());
        if (serviceResponse.isSuccess()) {
            return RestResponse.success(serviceResponse.getResult().getGoodsSearchEdit());
        }
        return RestResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
    }

    @RequestMapping("goods-selector-save")
    public RestResponse goodsSelectorSave(GoodsSelectorSaveQO goodsSelectorSaveQO) {
        GoodsSelectorDTO goodsSelectorDTO = EntityUtil.changeEntityByOrika(GoodsSelectorDTO.class, goodsSelectorSaveQO);
        goodsSelectorDTO.setEnterpriseId(getEnterpriseId());
        Long resourceId = UserDetailUtils.getUserDetail().getGoodsResourceInfo() == null ? null :
                UserDetailUtils.getUserDetail().getGoodsResourceInfo().getGoodsResourceId();
        ServiceResponse<GoodsRightsSelectorDTO> goodsRightsSelectorDtoServiceResponse = goodsRightsSelectorApiService.getGoodsRightsSelector(resourceId,
                getEnterpriseId(), UserDetailUtils.getUserDetail().getUserInfo().getSuperAdmin());
        if (!goodsRightsSelectorDtoServiceResponse.isSuccess()) {
            return RestResponse.failure(goodsRightsSelectorDtoServiceResponse.getCode(), goodsRightsSelectorDtoServiceResponse.getMessage());
        }
        GoodsRightsSelectorDTO goodsRightsSelectorDTO = goodsRightsSelectorDtoServiceResponse.getResult();
        goodsSelectorDTO.setHasRights(goodsRightsSelectorDTO.getHasRights());
        goodsSelectorDTO.setGoodsSearchRightValue(goodsRightsSelectorDTO.getGoodsRightsSearchValue());
        goodsSelectorDTO.setGoodsSearchRightText(goodsRightsSelectorDTO.getGoodsRightsSearchText());
        goodsSelectorDTO.setGoodsRightsBrands(goodsRightsSelectorDTO.getGoodsRightsBrands());
        goodsSelectorDTO.setGoodsRightsDomains(goodsRightsSelectorDTO.getGoodsRightsDomains());
        ServiceResponse<GoodsSelectorDTO> serviceResponse = goodsSelectorApiService.saveGoodsSelector(goodsSelectorDTO);
        if (serviceResponse.isSuccess()) {
            goodsSelectorDTO = serviceResponse.getResult();
            GoodsSelectorVO goodsSelectorVO = EntityUtil.changeEntityByOrika(GoodsSelectorVO.class, goodsSelectorDTO);
            return RestResponse.success(goodsSelectorVO);
        }
        return RestResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
    }

    @RequestMapping("goods-selector-show")
    public RestResponse goodsSelectorShow(GoodsSelectorSaveQO goodsSelectorSaveQO) {
        GoodsSelectorDTO goodsSelectorDTO = EntityUtil.changeEntityByOrika(GoodsSelectorDTO.class, goodsSelectorSaveQO);
        goodsSelectorDTO.setEnterpriseId(getEnterpriseId());
        Long resourceId = UserDetailUtils.getUserDetail().getGoodsResourceInfo() == null ? null :
                UserDetailUtils.getUserDetail().getGoodsResourceInfo().getGoodsResourceId();
        ServiceResponse<GoodsRightsSelectorDTO> goodsRightsSelectorDtoServiceResponse = goodsRightsSelectorApiService.getGoodsRightsSelector(resourceId,
                getEnterpriseId(), UserDetailUtils.getUserDetail().getUserInfo().getSuperAdmin());
        if (!goodsRightsSelectorDtoServiceResponse.isSuccess()) {
            return RestResponse.failure(goodsRightsSelectorDtoServiceResponse.getCode(), goodsRightsSelectorDtoServiceResponse.getMessage());
        }
        GoodsRightsSelectorDTO goodsRightsSelectorDTO = goodsRightsSelectorDtoServiceResponse.getResult();
        goodsSelectorDTO.setHasRights(goodsRightsSelectorDTO.getHasRights());
        goodsSelectorDTO.setGoodsSearchRightValue(goodsRightsSelectorDTO.getGoodsRightsSearchValue());
        goodsSelectorDTO.setGoodsSearchRightText(goodsRightsSelectorDTO.getGoodsRightsSearchText());
        ServiceResponse<GoodsSelectorDTO> serviceResponse = goodsSelectorApiService.showGoodsSelector(goodsSelectorDTO);
        if (serviceResponse.isSuccess()) {
            goodsSelectorDTO = serviceResponse.getResult();
            GoodsSelectorVO goodsSelectorVO = EntityUtil.changeEntityByOrika(GoodsSelectorVO.class, goodsSelectorDTO);
            return RestResponse.success(goodsSelectorVO);
        }
        return RestResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
    }


    @RequestMapping("change-goods-rights-for-selector")
    public RestResponse changeGoodsRightsForSelector(Long goodsSelectorId) {
        if (goodsSelectorId == null) {
            Long resourceId = UserDetailUtils.getUserDetail().getGoodsResourceInfo() == null ? null:UserDetailUtils.getUserDetail().getGoodsResourceInfo().getGoodsResourceId();
            ServiceResponse<GoodsRightsSelectorDTO> serviceResponse = goodsRightsSelectorApiService.getGoodsRightsSelector(resourceId,
                    getEnterpriseId(), UserDetailUtils.getUserDetail().getUserInfo().getSuperAdmin());
            if (serviceResponse.getResult() == null) {
                return RestResponse.failure(ErrorCodeEnums.MAll_ERR.getCode(), "没有权限");
            }
            GoodsRightsSelectorDTO goodsRightsSelectorDTO = new GoodsRightsSelectorDTO();
            goodsRightsSelectorDTO.setHasRights(serviceResponse.getResult().getHasRights());
            goodsRightsSelectorDTO.setGoodsRightsSearchText(serviceResponse.getResult().getGoodsRightsSearchText());
            goodsRightsSelectorDTO.setGoodsRightsSearchValue(serviceResponse.getResult().getGoodsRightsSearchValue());
            return RestResponse.success(goodsRightsSelectorDTO);
        }
        Long resourceId = UserDetailUtils.getUserDetail().getGoodsResourceInfo() == null ? null :
                UserDetailUtils.getUserDetail().getGoodsResourceInfo().getGoodsResourceId();
        ServiceResponse<GoodsRightsSelectorDTO> goodsRightsSelectorDtoServiceResponse = goodsRightsSelectorApiService.getGoodsRightsSelector(resourceId,
                getEnterpriseId(), UserDetailUtils.getUserDetail().getUserInfo().getSuperAdmin());
        if (!goodsRightsSelectorDtoServiceResponse.isSuccess()) {
            return RestResponse.failure(goodsRightsSelectorDtoServiceResponse.getCode(), goodsRightsSelectorDtoServiceResponse.getMessage());
        }
        if (goodsSelectorId == null) {
            GoodsRightsSelectorDTO goodsRightsSelectorDTO = new GoodsRightsSelectorDTO();
            goodsRightsSelectorDTO.setGoodsRightsSearchText(goodsRightsSelectorDtoServiceResponse.getResult().getGoodsRightsSearchText());
            goodsRightsSelectorDTO.setGoodsRightsSearchValue(goodsRightsSelectorDtoServiceResponse.getResult().getGoodsRightsSearchValue());
            goodsRightsSelectorDTO.setHasRights(goodsRightsSelectorDtoServiceResponse.getResult().getHasRights());
            return  RestResponse.success(goodsRightsSelectorDTO);
        }
        GoodsRightsSelectorDTO goodsRightsSelectorDTO = goodsRightsSelectorDtoServiceResponse.getResult();
        ServiceResponse<GoodsSelectorDTO> serviceResponse =  goodsSelectorApiService.updateGoodsSelectorRights(goodsSelectorId,
                goodsRightsSelectorDTO.getGoodsRightsSearchText(), goodsRightsSelectorDTO.getGoodsRightsSearchValue(), goodsRightsSelectorDTO.getHasRights(), goodsRightsSelectorDTO.getGoodsRightsBrands(), goodsRightsSelectorDTO.getGoodsRightsDomains(), getEnterpriseId());
        if (!serviceResponse.isSuccess()) {
            return RestResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
        }
        goodsRightsSelectorDTO = new GoodsRightsSelectorDTO();
        goodsRightsSelectorDTO.setGoodsRightsSearchText(serviceResponse.getResult().getGoodsSearchRightText());
        goodsRightsSelectorDTO.setGoodsRightsSearchValue(serviceResponse.getResult().getGoodsSearchRightValue());
        goodsRightsSelectorDTO.setHasRights(serviceResponse.getResult().getHasRights());
        return RestResponse.success(goodsRightsSelectorDTO);
    }



    @RequestMapping("goods-selector-update")
    public RestResponse goodsSelectorUpdate(GoodsSelectorUpdateQO goodsSelectorUpdateQO) {
        GoodsSelectorDTO goodsSelectorDTO = EntityUtil.changeEntityByOrika(GoodsSelectorDTO.class, goodsSelectorUpdateQO);
        goodsSelectorDTO.setEnterpriseId(getEnterpriseId());
        ServiceResponse<GoodsSelectorDTO> serviceResponse = goodsSelectorApiService.updateGoodsSelector(goodsSelectorDTO);
        if (serviceResponse.isSuccess()) {
            goodsSelectorDTO = serviceResponse.getResult();
            GoodsSelectorVO goodsSelectorVO = EntityUtil.changeEntityByOrika(GoodsSelectorVO.class, goodsSelectorDTO);
            return RestResponse.success(goodsSelectorVO);
        }
        return RestResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
    }

    @RequestMapping("check-selector-goods-hit")
    public RestResponse checkSelectorGoodsHit(Long goodsSelectorId, String goodsCode, String skuCode) {
        ServiceResponse<Boolean>  serviceResponse =
                goodsSelectorApiService.isSelectorHitGoods(goodsSelectorId, skuCode, goodsCode,  getEnterpriseId());
        return RestResponse.success(serviceResponse.getResult());
    }

}
